<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyAddServiceRequest;
use App\Http\Requests\StoreAddServiceRequest;
use App\Http\Requests\UpdateAddServiceRequest;
use App\Models\AddService;
use App\Models\Category;
use App\Models\ServiceSubCategory;
use App\Models\Shop;
use App\Models\User;
use Gate;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class AddServiceController extends Controller
{
    public function index(Request $request)
    {
        abort_if(Gate::denies('add_service_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $query = AddService::with(['category', 'subcategory', 'user', 'shops'])->select(sprintf('%s.*', (new AddService)->table));
            $table = Datatables::of($query);

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate      = 'add_service_show';
                $editGate      = 'add_service_edit';
                $deleteGate    = 'add_service_delete';
                $crudRoutePart = 'add-services';

                return view('partials.datatablesActions', compact(
                    'viewGate',
                    'editGate',
                    'deleteGate',
                    'crudRoutePart',
                    'row'
                ));
            });

            $table->editColumn('id', function ($row) {
                return $row->id ? $row->id : '';
            });
            $table->addColumn('category_name', function ($row) {
                return $row->category ? $row->category->name : '';
            });

            $table->addColumn('subcategory_name', function ($row) {
                return $row->subcategory ? $row->subcategory->name : '';
            });

            $table->addColumn('user_name', function ($row) {
                return $row->user ? $row->user->name : '';
            });

            $table->editColumn('status', function ($row) {
                return $row->status ? AddService::STATUS_SELECT[$row->status] : '';
            });
            $table->addColumn('shops_open_time', function ($row) {
                return $row->shops ? $row->shops->open_time : '';
            });

            $table->rawColumns(['actions', 'placeholder', 'category', 'subcategory', 'user', 'shops']);

            return $table->make(true);
        }

        return view('admin.addServices.index');
    }

    public function create()
    {
        abort_if(Gate::denies('add_service_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $categories = Category::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $subcategories = ServiceSubCategory::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $users = User::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $shops = Shop::pluck('open_time', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.addServices.create', compact('categories', 'shops', 'subcategories', 'users'));
    }

    public function store(StoreAddServiceRequest $request)
    {
        $addService = AddService::create($request->all());

        return redirect()->route('admin.add-services.index');
    }

    public function edit(AddService $addService)
    {
        abort_if(Gate::denies('add_service_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $categories = Category::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $subcategories = ServiceSubCategory::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $users = User::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $shops = Shop::pluck('open_time', 'id')->prepend(trans('global.pleaseSelect'), '');

        $addService->load('category', 'subcategory', 'user', 'shops');

        return view('admin.addServices.edit', compact('addService', 'categories', 'shops', 'subcategories', 'users'));
    }

    public function update(UpdateAddServiceRequest $request, AddService $addService)
    {
        $addService->update($request->all());

        return redirect()->route('admin.add-services.index');
    }

    public function show(AddService $addService)
    {
        abort_if(Gate::denies('add_service_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $addService->load('category', 'subcategory', 'user', 'shops');

        return view('admin.addServices.show', compact('addService'));
    }

    public function destroy(AddService $addService)
    {
        abort_if(Gate::denies('add_service_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $addService->delete();

        return back();
    }

    public function massDestroy(MassDestroyAddServiceRequest $request)
    {
        $addServices = AddService::find(request('ids'));

        foreach ($addServices as $addService) {
            $addService->delete();
        }

        return response(null, Response::HTTP_NO_CONTENT);
    }
}
