<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\MediaUploadingTrait;
use App\Http\Requests\MassDestroyServiceSubCategoryRequest;
use App\Http\Requests\StoreServiceSubCategoryRequest;
use App\Http\Requests\UpdateServiceSubCategoryRequest;
use App\Models\ServiceCategory;
use App\Models\ServiceSubCategory;
use Gate;
use Illuminate\Http\Request;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class ServiceSubCategoryController extends Controller
{
    use MediaUploadingTrait;

    public function index(Request $request)
    {
        abort_if(Gate::denies('service_sub_category_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $query = ServiceSubCategory::with(['service_category'])->select(sprintf('%s.*', (new ServiceSubCategory)->table));
            $table = Datatables::of($query);

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate      = 'service_sub_category_show';
                $editGate      = 'service_sub_category_edit';
                $deleteGate    = 'service_sub_category_delete';
                $crudRoutePart = 'service-sub-categories';

                return view('partials.datatablesActions', compact(
                    'viewGate',
                    'editGate',
                    'deleteGate',
                    'crudRoutePart',
                    'row'
                ));
            });

            $table->editColumn('id', function ($row) {
                return $row->id ? $row->id : '';
            });
            $table->editColumn('name', function ($row) {
                return $row->name ? $row->name : '';
            });
            $table->editColumn('slug', function ($row) {
                return $row->slug ? $row->slug : '';
            });
            $table->editColumn('icon', function ($row) {
                if ($photo = $row->icon) {
                    return sprintf(
                        '<a href="%s" target="_blank"><img src="%s" width="50px" height="50px"></a>',
                        $photo->url,
                        $photo->thumbnail
                    );
                }

                return '';
            });
            $table->editColumn('status', function ($row) {
                return $row->status ? ServiceSubCategory::STATUS_SELECT[$row->status] : '';
            });
            $table->addColumn('service_category_name', function ($row) {
                return $row->service_category ? $row->service_category->name : '';
            });

            $table->rawColumns(['actions', 'placeholder', 'icon', 'service_category']);

            return $table->make(true);
        }

        return view('admin.serviceSubCategories.index');
    }

    public function create()
    {
        abort_if(Gate::denies('service_sub_category_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $service_categories = ServiceCategory::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.serviceSubCategories.create', compact('service_categories'));
    }

    public function store(StoreServiceSubCategoryRequest $request)
    {
        $serviceSubCategory = ServiceSubCategory::create($request->all());

        if ($request->input('icon', false)) {
            $serviceSubCategory->addMedia(storage_path('tmp/uploads/' . basename($request->input('icon'))))->toMediaCollection('icon');
        }

        if ($media = $request->input('ck-media', false)) {
            Media::whereIn('id', $media)->update(['model_id' => $serviceSubCategory->id]);
        }

        return redirect()->route('admin.service-sub-categories.index');
    }

    public function edit(ServiceSubCategory $serviceSubCategory)
    {
        abort_if(Gate::denies('service_sub_category_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $service_categories = ServiceCategory::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $serviceSubCategory->load('service_category');

        return view('admin.serviceSubCategories.edit', compact('serviceSubCategory', 'service_categories'));
    }

    public function update(UpdateServiceSubCategoryRequest $request, ServiceSubCategory $serviceSubCategory)
    {
        $serviceSubCategory->update($request->all());

        if ($request->input('icon', false)) {
            if (! $serviceSubCategory->icon || $request->input('icon') !== $serviceSubCategory->icon->file_name) {
                if ($serviceSubCategory->icon) {
                    $serviceSubCategory->icon->delete();
                }
                $serviceSubCategory->addMedia(storage_path('tmp/uploads/' . basename($request->input('icon'))))->toMediaCollection('icon');
            }
        } elseif ($serviceSubCategory->icon) {
            $serviceSubCategory->icon->delete();
        }

        return redirect()->route('admin.service-sub-categories.index');
    }

    public function show(ServiceSubCategory $serviceSubCategory)
    {
        abort_if(Gate::denies('service_sub_category_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $serviceSubCategory->load('service_category');

        return view('admin.serviceSubCategories.show', compact('serviceSubCategory'));
    }

    public function destroy(ServiceSubCategory $serviceSubCategory)
    {
        abort_if(Gate::denies('service_sub_category_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $serviceSubCategory->delete();

        return back();
    }

    public function massDestroy(MassDestroyServiceSubCategoryRequest $request)
    {
        $serviceSubCategories = ServiceSubCategory::find(request('ids'));

        foreach ($serviceSubCategories as $serviceSubCategory) {
            $serviceSubCategory->delete();
        }

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function storeCKEditorImages(Request $request)
    {
        abort_if(Gate::denies('service_sub_category_create') && Gate::denies('service_sub_category_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $model         = new ServiceSubCategory();
        $model->id     = $request->input('crud_id', 0);
        $model->exists = true;
        $media         = $model->addMediaFromRequest('upload')->toMediaCollection('ck-media');

        return response()->json(['id' => $media->id, 'url' => $media->getUrl()], Response::HTTP_CREATED);
    }
}
